##=============================================================================
## Comparisons of the ordinary and random effects non linear models to
##                estimate plant disease progress curve.
## 
##                       Walmes Marques Zeviani,
##                           Rafaele Moreira,
##                         Larissa May De Mio,
##                     Paulo Justiniano Ribeiro Jr.
##=============================================================================

##-----------------------------------------------------------------------------
## Maça, Mancha foliar de Glomerella, Colletotrichum.

##-----------------------------------------------------------------------------
## Session definition.

require(lattice)
require(latticeExtra)
require(nlme)

rm(list=ls())
source("http://dl.dropboxusercontent.com/u/48140237/bandas.R")
source("/home/walmes/Dropbox/Public/bandas.R")

##-----------------------------------------------------------------------------
## Styles.

## display.brewer.all()
mycol <- c(brewer.pal(6, "Dark2"), brewer.pal(6, "Set1"))
old.tp <- trellis.par.get()
## names(old.tp)

ps <- list(box.rectangle=list(col=1, fill=c("gray70")),
           box.umbrella=list(col=1, lty=1),
           box.dot=list(pch="|"),
           dot.symbol=list(col=1),
           dot.line=list(col=1, lty=3),
           plot.symbol=list(col=1, cex=0.7),
           plot.line=list(col=1),
           plot.polygon=list(col="gray80"),
           superpose.line=list(col=mycol),
           superpose.symbol=list(col=mycol),
           superpose.polygon=list(col=mycol),
           strip.background=list(col=c("gray90","gray50")))
## show.settings()
## show.settings(ps)
trellis.par.set(ps)

##-----------------------------------------------------------------------------
## Reading data.

da <- read.table("glomerella.txt", header=TRUE)
da$ar <- factor(da$ar)
str(da)

##-----------------------------------------------------------------------------
## Reordering levels by maximum value.

da$ar <- factor(da$ar, levels=names(sort(with(da, tapply(y, ar, max)))))

##-----------------------------------------------------------------------------
## Ploting data.

p1 <- 
    xyplot(y~x|ar, layout=c(5,6), as.table=TRUE,
           ylab="Severity", xlab="Days",
           data=da, strip=FALSE, type="o"); p1

p2 <- 
    xyplot(y~x, groups=ar, as.table=TRUE,
           ylab="Severity", xlab="Days",
           data=da, strip=FALSE, type="o", lty=3)+
    as.layer(xyplot(y~x, da, type="a", lwd=2)); p2


pdf("init.pdf", w=8, h=4); trellis.par.set(ps)
## plot(p1, split=c(1,1,1,2), more=TRUE)
## plot(p2, split=c(1,2,1,2), more=FALSE)
plot(p1, split=c(1,1,2,1), more=TRUE)
plot(p2, split=c(2,1,2,1), more=FALSE)
dev.off()

##-----------------------------------------------------------------------------
## Frequencies.

tb <- xtabs(~ar, data=da); tb
range(tb)

##-----------------------------------------------------------------------------
## Fitting the naive model.

plot(y~x, da)
mystart <- list(I=1, A=30, x0=55, S=7)
with(mystart, curve(I+(A-I)/(1+exp(-(x-x0)/S)), add=TRUE, col=2))

n0 <- nls(y~I+A/(1+exp(-(x-x0)/S)), data=da,
          start=mystart, trace=TRUE)

## parameter estimates and confidence intervals
summary(n0)
confint.default(n0)

##-----------------------------------------------------------------------------
## Fitting the non linear random effects model.

dd <- groupedData(y~x|ar, data=da, order.groups=FALSE)

n00 <- nlme(y~I+A/(1+exp(-(x-x0)/S)),
            start=coef(n0), fixed=I+A+x0+S~1,
            random=A+S~1, data=dd) # converge

summary(n00)

##-----------------------------------------------------------------------------
## Model comparisons.

## estimates and standard errors
summary(n0)$coeff
summary(n00)$tTable

## confidence intervals
ci0 <- cbind(confint.default(n0), coef(n0))
ci00 <- intervals(n00)$fixed

ci00 <- ci00[,c(2,1,3)]
ci0 <- ci0[,c(3,1,2)]
colnames(ci0) <- colnames(ci00) <- c("est","lwr","upr")

ci <- as.data.frame(rbind(ci0, ci00))
ci$par <- factor(rownames(ci), levels=c("I","x0","S","A"))
rownames(ci) <- NULL
ci$model <- gl(2, 4, labels=c("nls","nlme"))
ci

pdf("ic.pdf", w=8, h=2.5); trellis.par.set(ps)
segplot(model~lwr+upr|par, data=ci,
        scales=list(x="free"), layout=c(NA,1),
        draw.bands=FALSE, centers=est,
        ylab="Model",
        xlab=list(c(
            "Lower asymptote    ",
            "Inflexion  ",
            "  Scale",
            "   Upper asymptote")),
        segments.fun=panel.arrows, ends="both", 
        angle=90, length=1, unit="mm")
dev.off()

##-----------------------------------------------------------------------------
## Random effects.

a <- stack(ranef(n00))

p3 <- splom(ranef(n00), xlab=NULL)+
    layer(panel.abline(v=0, h=0, lty=2))
p3

p4 <- qqmath(~values|ind, data=a, scales="free",
             xlab="Standard normal",
             ylab="Predicted random effects")
p4

pdf("ranef.pdf", w=8, h=4); trellis.par.set(ps)
plot(p3, split=c(1,1,2,1), more=TRUE)
plot(p4, split=c(2,1,2,1), more=FALSE)
dev.off()

##-----------------------------------------------------------------------------
## Residuals plot.

da$r0 <- residuals(n0)
da$f0 <- fitted(n0)
da$r00 <- residuals(n00, type="pearson")
da$f00 <- fitted(n00)

## qqmath(~r0+r00|ar, da, strip=FALSE, auto.key=TRUE)
## qqmath(~r0+r00, da, strip=FALSE, auto.key=TRUE)

p5 <- 
    xyplot(r0~f0, da, strip=FALSE, col=1,
           ylab="Residuals", xlab="Fitted values",
           key=list(columns=2, text=list(c("nls","nlme")),
               points=list(col=1:2, pch=1)),
           xlim=extendrange(range(da$f0, da$f00)))+
    as.layer(xyplot(r00~f00, da, col=2))
p5

p6 <- 
    xyplot(sqrt(abs(r0))~f0, da, strip=FALSE,
           col=1, type=c("p","r"),
           key=list(columns=2, text=list(c("nls","nlme")),
               points=list(col=1:2, pch=1)),
           ylab=expression(sqrt("|"*Residuals*"|")), xlab="Fitted values",
           xlim=extendrange(range(da$f0, da$f00)))+
    as.layer(xyplot(sqrt(abs(r00))~f00, da,
                    col=2, type=c("p","r")))
p6

pdf("res.pdf", w=8, h=4); trellis.par.set(ps)
plot(p5, split=c(1,1,2,1), more=TRUE)
plot(p6, split=c(2,1,2,1), more=FALSE)
dev.off()

##-----------------------------------------------------------------------------
## Plot of fitted values with confidence bands.

pred <- expand.grid(ar=levels(da$ar), x=0:90)
str(pred)
pred$y0 <- predict(n0, newdata=pred)            # naive
pred$y00 <- predict(n00, newdata=pred, level=0) # populacional
pred$y01 <- predict(n00, newdata=pred, level=1) # individual

model <- deriv3(~I+A/(1+exp(-(x-x0)/S)),
                c("I","A","x0","S"), function(x, I, A, x0, S){NULL})
pred2 <- data.frame(x=0:90)

m0 <- do.call(model, args=c(list(x=pred2$x), as.list(coef(n0))))
m00 <- do.call(model, args=c(list(x=pred2$x), as.list(fixef(n00))))

F0 <- attr(m0, "gradient")
F00 <- attr(m00, "gradient")
dim(F0)

U0 <- chol(vcov(n0))
U00 <- chol(vcov(n00))

pred2$se0 <- sqrt(apply(F0%*%t(U0), 1, function(x) sum(x^2)))
pred2$se00 <- sqrt(apply(F00%*%t(U00), 1, function(x) sum(x^2)))

zval <- qnorm(p=c(lwr=0.025, fit=0.5, upr=0.975))
me <- outer(pred2$se0, zval, "*")
b <- sweep(me, 1, m0, "+")
colnames(b) <- paste(colnames(b), "0", sep="")
pred2 <- cbind(pred2, b)

me <- outer(pred2$se00, zval, "*")
b <- sweep(me, 1, m00, "+")
colnames(b) <- paste(colnames(b), "00", sep="")
pred2 <- cbind(pred2, b)

str(pred2)

##-----------------------------------------------------------------------------
## Plots.

col <- 1
lwd <- c(1,1.5,1)
lty <- c(2,1,1)
nomes <- c("nls","nlme-pop","nlme-ind")
xlab <- "Days after first evaluation"
ylab <- "Severity"

## populational
pdf("r1.pdf", w=8, h=5); trellis.par.set(ps)
xyplot(y~x, data=da, groups=ar, type="p", col="gray50",
       xlab=xlab, ylab=ylab,
       key=list(corner=c(0.02,0.98), columns=1,
           lines=list(lwd=lwd[1:2], col=col, lty=lty[1:2]),
           text=list(nomes[1:2])))+
    as.layer(xyplot(fit0~x, data=pred2, type="l",
                    col=col, lty=lty[1], lwd=lwd[1],
                    prepanel=prepanel.cbH, cty="bands",
                    ly=pred2$lwr0, uy=pred2$upr0,
                    panel=panel.cbH
                    ))+
    as.layer(xyplot(fit00~x, data=pred2, type="l",
                    col=col, lty=lty[2], lwd=lwd[2],
                    prepanel=prepanel.cbH, cty="bands",
                    ly=pred2$lwr00, uy=pred2$upr00,
                    panel=panel.cbH
                    ))
dev.off()

## individuals
pdf("r2.pdf", w=8, h=5); trellis.par.set(ps)
xyplot(y01~x|ar, data=pred, type="l", strip=FALSE,
       xlab=xlab, ylab=ylab,
       col=col, lty=lty[3], lwd=lwd[3],
       key=list(columns=3,
           lines=list(lwd=lwd, col=col, lty=lty),
           text=list(nomes)))+
    as.layer(xyplot(fit0~x, data=pred2, type="l",
                    col=col, lty=lty[1], lwd=lwd[1],
                    prepanel=prepanel.cbH, cty="bands",
                    ly=pred2$lwr0, uy=pred2$upr0,
                    panel=panel.cbH
                    ))+
    as.layer(xyplot(fit00~x, data=pred2, type="l",
                    col=col, lty=lty[2], lwd=lwd[2],
                    prepanel=prepanel.cbH, cty="bands",
                    ly=pred2$lwr00, uy=pred2$upr00,
                    panel=panel.cbH
                    ))+
    as.layer(xyplot(y~x|ar, data=da, groups=ar, type="p", col="gray50"))
dev.off()

## mixed
pdf("r3.pdf", w=8, h=5); trellis.par.set(ps)
xyplot(y01~x, groups=ar, data=pred, type="l",
       xlab=xlab, ylab=ylab,
       lty=3, lwd=1, col=1,
       key=list(corner=c(0.02,0.98), columns=1,
           lines=list(lwd=3, col=c("red","blue"), lty=1),
           text=list(nomes[1:2])))+
    as.layer(xyplot(y~x, data=da, groups=ar, type="p",
                    xlab=xlab, ylab=ylab, col=1))+
    as.layer(xyplot(fit0~x, data=pred2, type="l",
                    col="red", lty=1, lwd=3,
                    prepanel=prepanel.cbH, cty="bands",
                    fill="red", alpha=0.3,
                    ly=pred2$lwr0, uy=pred2$upr0,
                    panel=panel.cbH
                    ))+
    as.layer(xyplot(fit00~x, data=pred2, type="l",
                    col="blue", lty=1, lwd=3,
                    prepanel=prepanel.cbH, cty="bands",
                    fill="blue", alpha=0.3,
                    ly=pred2$lwr00, uy=pred2$upr00,
                    panel=panel.cbH
                    ))
dev.off()

##-----------------------------------------------------------------------------

save.image("nlmeaucpd.RData")


##-----------------------------------------------------------------------------
