##=============================================================================
##   Análise da área abaixo da curva de crescimento como função da
##           temperatura do ambiente para desenvolvimento.
##
##                      Walmes Marques Zeviani,
##                          Eliane Rogoski,
##                        Larissa May De Mio,
##                    Paulo Justiniano Ribeiro Jr.
##=============================================================================

##-----------------------------------------------------------------------------
## Session definition.

require(lattice)
require(latticeExtra)
require(reshape)
require(plyr)
require(nlme)

rm(list=ls())
## source("http://dl.dropboxusercontent.com/u/48140237/aac.R")
## source("http://dl.dropboxusercontent.com/u/48140237/bandas.R")
source("/home/walmes/Dropbox/Public/aac.R")
source("/home/walmes/Dropbox/Public/bandas.R")

##-----------------------------------------------------------------------------
## Styles.

## display.brewer.all()
mycol <- c(brewer.pal(6, "Dark2"), brewer.pal(6, "Set1"))
old.tp <- trellis.par.get()
## names(old.tp)

ps <- list(box.rectangle=list(col=1, fill=c("gray70")),
           box.umbrella=list(col=1, lty=1),
           box.dot=list(pch="|"),
           dot.symbol=list(col=1),
           dot.line=list(col=1, lty=3),
           plot.symbol=list(col=1, cex=0.7),
           plot.line=list(col=1),
           plot.polygon=list(col="gray80"),
           superpose.line=list(col=mycol),
           superpose.symbol=list(col=mycol),
           superpose.polygon=list(col=mycol),
           strip.background=list(col=c("gray90","gray50")))
## show.settings()
## show.settings(ps)
trellis.par.set(ps)

##-----------------------------------------------------------------------------
## Reading data.

db <- read.table("allexp.txt", header=TRUE)
str(db)

dc <- melt(db, measure.vars=grep("^X\\d", names(db)), variable_name="dia")
names(dc) <- tolower(names(dc))
str(dc)

dc$dia <- as.numeric(gsub("\\D", "", dc$dia))
dc$ue <- with(dc, interaction(exp, temp, rep, folha, drop=TRUE))
str(dc)

##-----------------------------------------------------------------------------
## Plots.

xyplot(value~dia, data=dc)

pdf("profiles.pdf", w=8, h=6); trellis.par.set(ps)
xyplot(value~dia|factor(temp), groups=ue, data=dc, as.table=TRUE,
       layout=c(NA,2), xlab="Days", ylab="Diameter", type="o", cex=0.5)
dev.off()

##-----------------------------------------------------------------------------
## Area under the growth curve.

dd <- ddply(dc, .(exp, temp, rep, folha), summarise,
            aa=aac(dia, value))
dd$ua <- with(dd, interaction(exp, temp, rep, drop=TRUE))
str(dd)

## dd <- subset(dd, aa<2500)

##-----------------------------------------------------------------------------
## Frequency.

## zero values (no grow)
xtabs(~(aa==0), data=dd)

## unique temp values
sort(unique(dd$temp))

## temps in each experiment
xtabs(~temp+exp, data=dd)

##-----------------------------------------------------------------------------
## The experiment works as a blocking factor. The effect of temperature
## can consider the generalized beta model but has the infringement of
## variances assumption contained in the natural scale. By taking the
## log variances are more homogeneous and the effect of temperature is
## almost quadratic. The log transformation drawback is that zero values
## are tranformed to -inf, so, power transformation are more suitable.

##-----------------------------------------------------------------------------
## Ver.

## tranformations?
xlab <- "Temperature"
ylab <- "Size"

p1 <- xyplot((aa)~temp, type=c("p","a"),
             data=dd, jitter.x=TRUE,
             xlab=xlab, ylab=ylab, main="Original scale"); p1

p2 <- xyplot(log(aa)~temp, type=c("p","a"),
             data=subset(dd, aa>0), jitter.x=TRUE,
             xlab=xlab, ylab=ylab, main="Log scale"); p2

p3 <- xyplot((aa^(1/2))~temp, type=c("p","a"),
             data=dd, jitter.x=TRUE,
             xlab=xlab, ylab=ylab, main="Square root scale"); p3

p4 <- xyplot((aa^(1/3))~temp, type=c("p","a"),
             data=dd, jitter.x=TRUE,
             xlab=xlab, ylab=ylab, main="Cubic root scale"); p4

## pdf("scales.pdf", w=8, h=4); trellis.par.set(ps)
## plot(p1, split=c(1,1,4,1), more=TRUE)
## plot(p2, split=c(2,1,4,1), more=TRUE)
## plot(p3, split=c(3,1,4,1), more=TRUE)
## plot(p4, split=c(4,1,4,1), more=FALSE)
## dev.off()

pdf("scales.pdf", w=8, h=6); trellis.par.set(ps)
plot(p1, split=c(1,1,2,2), more=TRUE)
plot(p2, split=c(2,1,2,2), more=TRUE)
plot(p3, split=c(1,2,2,2), more=TRUE)
plot(p4, split=c(2,2,2,2), more=FALSE)
dev.off()

## effect of experimental unit?
pdf("eueffect.pdf", w=8, h=6); trellis.par.set(ps)
xyplot((aa^(1/3))~temp,
       xlab="Temperature", ylab="Cubic root of AUGC",
       groups=ua, type=c("p","a"),
       data=subset(dd),
       panel=function(x, y, groups, ...){
           panel.xyplot(x, y, groups=groups, ...)
           d <- 0.5
           xm <- ave(x, groups)+d
           ym <- ave(y, groups)
           panel.segments(x, y, xm, ym, col="gray50")
           da <- ddply(data.frame(x=x, y=y, groups=groups),
                       .(groups, x), summarise, y=mean(y))
           panel.points(da$x+d, da$y, pch=19)
           panel.abline(v=unique(da$x), lty=3, col="gray30")
       })
dev.off()

##-----------------------------------------------------------------------------
## The random effect will accommodate variations between experimental
## units while the variance function will accommodate the fact that the
## variance go to zero when the average goes to zero. With that gives to
## use the data in the original scale. Before it is advisable to check
## the variance average relation.

nlevels(dd$ua)
mv <- ddply(dd, .(ua), summarise, m=mean(aa), s=var(aa), n=length(aa))

## original
xyplot(s~m, data=mv, type=c("p","r"))

## log
pdf("loglog.pdf", w=8, h=6); trellis.par.set(ps)
xyplot(s~m, data=subset(mv, s>0 & m>0),
       xlab="log of sample variance", ylab="log of sample mean",
       scales=list(log=TRUE), type=c("p","r"))
dev.off()

##-----------------------------------------------------------------------------
## Naive model.

dd$y <- dd$aa/100
de <- groupedData(y~temp|ua, data=dd, order=FALSE)

n0 <- gnls(y~thy*exp(thq*(temp-thx)^2+thc*(temp-thx)^3),
           data=de, start=c(thy=10, thq=-0.08, thx=22, thc=-0.01),
           params=thy+thq+thx+thc~1)
summary(n0)

n1 <- nlme(y~thy*exp(thq*(temp-thx)^2+thc*(temp-thx)^3),
           data=de, start=c(thy=7, thq=-0.2, thx=22, thc=-0.01),
           random=thy~1, fixed=thy+thq+thx+thc~1)
summary(n1)

anova(n1, n0)

##-----------------------------------------------------------------------------
## Observed vs fitted.

plot(y~temp, dd)
points(fitted(n1)~I(dd$temp+0.25), pch=19)
with(as.list(coef(n0)),
     curve(thy*exp(thq*(x-thx)^2+thc*(x-thx)^3), add=TRUE, col=2))
with(as.list(fixef(n1)),
     curve(thy*exp(thq*(x-thx)^2+thc*(x-thx)^3), add=TRUE, col=3))

##-----------------------------------------------------------------------------
## Assumptions.

qqnorm(residuals(n0))
qqnorm(residuals(n0, type="pearson"))

plot(residuals(n0)~fitted(n0))
plot(residuals(n0, "pearson")~fitted(n0))

##-----------------------------------------------------------------------------
## Model with variance modelling.

n2 <- gnls(y~thy*exp(thq*(temp-thx)^2+thc*(temp-thx)^3),
           data=de, start=c(thy=7, thq=-0.2, thx=22, thc=-0.01),
           params=thy+thq+thx+thc~1,
           weights=varComb(varIdent(form=~1),
               varPower(form=~fitted(.))))
summary(n2)

n3 <- nlme(y~thy*exp(thq*(temp-thx)^2+thc*(temp-thx)^3),
           data=de, start=c(thy=1.5, thq=-0.2, thx=22, thc=0.005),
           random=thy~1, fixed=thy+thq+thx+thc~1,
           weights=varComb(varIdent(form=~1), varPower(value=0.7,
               form=~fitted(.))))
summary(n3)

##-----------------------------------------------------------------------------
## LRT.

anova(n1, n0) ## null random effects withou modelling variance?
anova(n3, n2) ## null random effects with modelling variance?
anova(n3, n1) ## null variance function?

##-----------------------------------------------------------------------------
## Segplot.

ci <- rbind(
    intervals(n0)$coef[,c(2,1,3)],
    intervals(n1)$fixed[,c(2,1,3)],
    intervals(n2)$coef[,c(2,1,3)],
    intervals(n3)$fixed[,c(2,1,3)])
ci <- as.data.frame(ci)
ci$par <- factor(rownames(ci), levels=c("thx","thy","thq","thc"))
rownames(ci) <- NULL
ci$model <- gl(4, 4, labels=c("nls","nlme","nls-v","nlme-v"))
str(ci)

pdf("icvar.pdf", w=8, h=2.5); trellis.par.set(ps)
segplot(model~lower+upper|par, data=ci,
        scales=list(x="free"), layout=c(NA,1),
        draw.bands=FALSE, centers=est.,
        ylab="Model",
        xlab=list(c(
            "Optimum temperature  ",
            "Size at optimum ",
            " Curvature",
            "  Skewness")),
        segments.fun=panel.arrows, ends="both", 
        angle=90, length=1, unit="mm")+
    layer(panel.abline(v=0, lty=2))
dev.off()

##-----------------------------------------------------------------------------
## Assumptions.

## qqnorm(residuals(n2))
## qqnorm(residuals(n2, type="pearson"))

pdf("resid.pdf", w=8, h=6); trellis.par.set(ps)
xyplot(sqrt(abs(residuals(n1, type="pearson")))~fitted(n1),
       type=c("p","r"), col=1,
       ylab=expression(sqrt("|"*Residuals*"|")), xlab="Fitted values",
       key=list(columns=2, text=list(c("nlme","nlme-v")),
               points=list(col=1:2, pch=1)))+
    as.layer(xyplot(sqrt(abs(residuals(n3, type="pearson")))~fitted(n3),
                    type=c("p","r"), col=2))
dev.off()

##-----------------------------------------------------------------------------
## Quais são as raízes para um crescimento de q = 0.05 do ótimo?

## thy*q  = thy*exp(thq*z^2+thc*z^3), z = x-thx
##     q  =     exp(thq*z^2+thc*z^3)
## log(q) =         thq*z^2+thc*z^3

## th <- fixef(n2)
## r <- polyroot(z=c(-log(0.05), 0, th["thq"], th["thc"]))+th["thx"]

col <- c("red","blue")
lwd <- c(1.5,1.5)
lty <- c(1,1)
alpha <- c(0.25,0.25)
nomes <- c("nlme","nlme-v")
xlab <- "Temperature"
ylab <- "AUGC"

cutt <- 0.1

pdf("cuts.pdf", w=8, h=5); trellis.par.set(ps)
xyplot(y~temp, data=dd,
       xlab="Temperature", ylab="AUGC",
       key=list(corner=c(0.98,0.98), columns=1,
           lines=list(lwd=lwd[1:2], col=col, lty=lty[1:2]),
           text=list(nomes[1:2])))
trellis.focus("panel", 1, 1, h=FALSE)
with(as.list(fixef(n1)),
     {
         col <- 2
         panel.curve(thy*exp(thq*(x-thx)^2+thc*(x-thx)^3), add=TRUE, col=col)
         panel.abline(h=c(1,cutt)*thy, col=col, lty=2)
         panel.segments(thx, thy, thx, 0, col=col, lty=1, lwd=2)
         r <- polyroot(z=c(-log(cutt), 0, thq, thc))+thx
         panel.abline(v=r, col=col, lty=2)
     })
with(as.list(fixef(n3)),
     {
         col <- 4
         panel.curve(thy*exp(thq*(x-thx)^2+thc*(x-thx)^3), add=TRUE, col=col)
         panel.abline(h=c(1,cutt)*thy, col=col, lty=2)
         panel.segments(thx, thy, thx, 0, col=col, lty=1, lwd=2)
         r <- polyroot(z=c(-log(cutt), 0, thq, thc))+thx
         panel.abline(v=r, col=col, lty=2)
     })
trellis.unfocus()
dev.off()

##-----------------------------------------------------------------------------
## Plot of fitted values with confidence bands.

pred <- expand.grid(temp=seq(10,32,by=0.1))
str(pred)
pred$y00 <- predict(n1, newdata=pred, level=0) # populacional
pred$y01 <- predict(n3, newdata=pred, level=0) # individual

model <- deriv3(~thy*exp(thq*(temp-thx)^2+thc*(temp-thx)^3),
                c("thy","thq","thx","thc"),
                function(temp, thy,thq,thx,thc){NULL})
pred2 <- pred

m0 <- do.call(model, args=c(list(temp=pred2$temp),
                         as.list(fixef(n1))))
m00 <- do.call(model, args=c(list(temp=pred2$temp),
                          as.list(fixef(n3))))

F0 <- attr(m0, "gradient")
F00 <- attr(m00, "gradient")
dim(F0)

U0 <- chol(vcov(n1))
U00 <- chol(vcov(n3))

pred2$se0 <- sqrt(apply(F0%*%t(U0), 1, function(x) sum(x^2)))
pred2$se00 <- sqrt(apply(F00%*%t(U00), 1, function(x) sum(x^2)))

zval <- qnorm(p=c(lwr=0.025, fit=0.5, upr=0.975))
me <- outer(pred2$se0, zval, "*")
b <- sweep(me, 1, m0, "+")
colnames(b) <- paste(colnames(b), "0", sep="")
pred2 <- cbind(pred2, b)

me <- outer(pred2$se00, zval, "*")
b <- sweep(me, 1, m00, "+")
colnames(b) <- paste(colnames(b), "00", sep="")
pred2 <- cbind(pred2, b)

str(pred2)

##-----------------------------------------------------------------------------
## Plots.

## populational
pdf("bands.pdf", w=8, h=5); trellis.par.set(ps)
xyplot(y~temp, data=dd, type="p", col="gray50",
       xlab=xlab, ylab=ylab,
       key=list(corner=c(0.02,0.98), columns=1,
           lines=list(lwd=lwd[1:2], col=col, lty=lty[1:2]),
           text=list(nomes[1:2])))+
    as.layer(xyplot(fit0~temp, data=pred2, type="l",
                    col=col[1], lty=lty[1], lwd=lwd[1],
                    prepanel=prepanel.cbH, cty="bands",
                    ly=pred2$lwr0, uy=pred2$upr0,
                    fill=col[1], alpha=alpha[1],
                    panel=panel.cbH
                    ))+
    as.layer(xyplot(fit00~temp, data=pred2, type="l",
                    col=col[2], lty=lty[2], lwd=lwd[2],
                    prepanel=prepanel.cbH, cty="bands",
                    ly=pred2$lwr00, uy=pred2$upr00,
                    fill=col[2], alpha=alpha[2],
                    panel=panel.cbH
                    ))
dev.off()

##-----------------------------------------------------------------------------

save.image("sizetemp.RData")

##-----------------------------------------------------------------------------
